package com.sample;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.fail;

import java.io.InputStreamReader;
import java.io.Reader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.SystemEventListenerFactory;
import org.drools.audit.WorkingMemoryInMemoryLogger;
import org.drools.audit.event.LogEvent;
import org.drools.audit.event.RuleFlowNodeLogEvent;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.KnowledgeBuilderResult;
import org.drools.builder.ResourceType;
import org.drools.builder.ResultSeverity;
import org.drools.io.ResourceFactory;
import org.drools.runtime.Environment;
import org.drools.runtime.EnvironmentName;
import org.drools.runtime.StatefulKnowledgeSession;
import org.jbpm.process.audit.JPAProcessInstanceDbLog;
import org.jbpm.process.workitem.wsht.SyncWSHumanTaskHandler;
import org.jbpm.task.Group;
import org.jbpm.task.TaskService;
import org.jbpm.task.User;
import org.jbpm.task.service.local.LocalTaskService;
import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.Before;
import org.junit.BeforeClass;

import bitronix.tm.TransactionManagerServices;
import bitronix.tm.resource.jdbc.PoolingDataSource;

public abstract class JbpmPersistenceTestCase {
    protected static PoolingDataSource jbpmDS;
    protected static KnowledgeBase kbase;

    protected EntityManagerFactory emf;
    protected EntityManagerFactory emf2;
    public static String[] files;

    private org.jbpm.task.service.TaskService myTaskService;
    private WorkingMemoryInMemoryLogger logger;
    
    @BeforeClass
    public static void beforeClass() {
        useMySQL();
        //useHSQL();
    }

    public static void useHSQL() {
        jbpmDS = new PoolingDataSource();
        jbpmDS.setMaxPoolSize(3);
        jbpmDS.setAllowLocalTransactions(true);
        jbpmDS.setUniqueName("jdbc/jbpmDS");
        jbpmDS
                .setClassName("bitronix.tm.resource.jdbc.lrc.LrcXADataSource");
        jbpmDS.getDriverProperties().put("driverClassName",
                "org.hsqldb.jdbcDriver");
        jbpmDS.getDriverProperties().put("url",
                "jdbc:hsqldb:file:/home/jsvitak/bitronixDB");
        jbpmDS.init();
    }

    public static void useMySQL() {
        jbpmDS = new PoolingDataSource();
        jbpmDS.setMaxPoolSize(3);
        jbpmDS.setAllowLocalTransactions(true);
        jbpmDS.setUniqueName("jdbc/jbpmDS");
        jbpmDS.setClassName("bitronix.tm.resource.jdbc.lrc.LrcXADataSource");
        jbpmDS.getDriverProperties().put("driverClassName",
                "com.mysql.jdbc.Driver");
        jbpmDS
                .getDriverProperties()
                .put("url",
                        "jdbc:mysql://localhost:3306/brms");
        jbpmDS.getDriverProperties().put("user", "brms");
        jbpmDS.getDriverProperties().put("password", "brms");
        jbpmDS.init();
    }

    @Before
    public void before() {
        System.setProperty("java.naming.factory.initial",
                "bitronix.tm.jndi.BitronixInitialContextFactory");
        System.setProperty("jbpm.usergroup.callback", "org.jbpm.task.service.DefaultUserGroupCallbackImpl");
        emf = Persistence
                .createEntityManagerFactory("org.jbpm.persistence.jpa.bitronix");
        emf2 = Persistence.createEntityManagerFactory("org.jbpm.task");

        KnowledgeBuilder kbuilder = KnowledgeBuilderFactory
                .newKnowledgeBuilder();
        for (String file : files) {
            kbuilder.add(ResourceFactory.newClassPathResource(file),
                    ResourceType.BPMN2);
        }
        for (KnowledgeBuilderResult result : kbuilder.getResults(
                ResultSeverity.ERROR, ResultSeverity.WARNING)) {
            System.err.println(result.toString());
        }
        if (kbuilder.hasErrors())
            Assert.fail("Found errors in process!");

        kbase = KnowledgeBaseFactory.newKnowledgeBase();
        kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());
    }

    protected Environment getEnvironment() {
        Environment result = KnowledgeBaseFactory.newEnvironment();
        result.set(EnvironmentName.ENTITY_MANAGER_FACTORY, emf);
        result.set(EnvironmentName.TRANSACTION_MANAGER,
                TransactionManagerServices.getTransactionManager());

        return result;
    }

    public TaskService getTaskService(StatefulKnowledgeSession ksession) {
        if (myTaskService == null) {
            myTaskService = new org.jbpm.task.service.TaskService(emf2,
                    SystemEventListenerFactory.getSystemEventListener());
        }
        LocalTaskService localTaskService = new LocalTaskService(myTaskService);

        MockUserInfo userInfo = new MockUserInfo();
        myTaskService.setUserinfo(userInfo);

        Map vars = new HashMap();
        Reader reader = new InputStreamReader(this.getClass().getResourceAsStream("/LoadUsers.mvel"));     
        Map<String, User> users = ( Map<String, User> ) org.jbpm.task.service.TaskService.eval( reader, vars );
        reader = new InputStreamReader(this.getClass().getResourceAsStream("/LoadGroups.mvel"));
        Map<String, Group> groups = ( Map<String, Group> ) org.jbpm.task.service.TaskService.eval( reader, vars ); 
        myTaskService.addUsersAndGroups(users, groups);

        SyncWSHumanTaskHandler humanTaskHandler = new SyncWSHumanTaskHandler(
                localTaskService, ksession);
        humanTaskHandler.setLocal(true);
        humanTaskHandler.connect();
        ksession.getWorkItemManager().registerWorkItemHandler("Human Task",
                humanTaskHandler);

        logger = new WorkingMemoryInMemoryLogger(ksession);
        return localTaskService;
    }
    
    public void assertNodeTriggered(long processInstanceId, String... nodeNames) {
        List<String> names = new ArrayList<String>();
        for (String nodeName: nodeNames) {
            names.add(nodeName);
        }
        /*if (sessionPersistence) {
            List<NodeInstanceLog> logs = log.findNodeInstances(processInstanceId);
            if (logs != null) {
                for (NodeInstanceLog l: logs) {
                    String nodeName = l.getNodeName();
                    if (l.getType() == NodeInstanceLog.TYPE_ENTER && names.contains(nodeName)) {
                        names.remove(nodeName);
                    }
                }
            }
            
        } else {*/
            for (LogEvent event: logger.getLogEvents()) {
                if (event instanceof RuleFlowNodeLogEvent) {
                    String nodeName = ((RuleFlowNodeLogEvent) event).getNodeName();
                    if (names.contains(nodeName)) {
                        names.remove(nodeName);
                    }
                }
            }
        /*}*/
        if (!names.isEmpty()) {
            String s = names.get(0);
            for (int i = 1; i < names.size(); i++) {
                s += ", " + names.get(i);
            }
            fail("Node(s) not executed: " + s);
        }
    }
    
    public void assertProcessInstanceActive(long processInstanceId, StatefulKnowledgeSession ksession) {
        assertNotNull(ksession.getProcessInstance(processInstanceId));
    }
    public void assertProcessInstanceCompleted(long processInstanceId, StatefulKnowledgeSession ksession) {
        assertNull(ksession.getProcessInstance(processInstanceId));
    }
    
    @AfterClass
    public static void afterClass() {
        if (jbpmDS != null)
            jbpmDS.close();
    }
}
