package com.sample;

import static org.junit.Assert.assertEquals;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.drools.persistence.jpa.JPAKnowledgeService;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.process.ProcessInstance;
import org.jbpm.task.Status;
import org.jbpm.task.TaskService;
import org.jbpm.task.query.TaskSummary;
import org.junit.Test;

// https://bugzilla.redhat.com/show_bug.cgi?id=871899
// needs empty MySQL database
public class GetTasksByProcessIdTest extends JbpmPersistenceTestCase {
    
    public GetTasksByProcessIdTest() {
        super();
        files = new String[] { "com.sample.GetTasksByProcessId.bpmn2" };
    }

    @Test
    public void testTasksByStatusByProcessIdAndByTaskName() {
        List<TaskSummary> list = new ArrayList<TaskSummary>();
        StatefulKnowledgeSession ksession = JPAKnowledgeService.newStatefulKnowledgeSession(kbase, null, getEnvironment());
        
        // my local task service
        TaskService taskService = getTaskService(ksession);
        
        ProcessInstance processInstance = ksession.startProcess("com.sample.GetTasksByProcessId");
        System.out.println("Process Instance Id =  "+ processInstance.getId());
        
        long id = processInstance.getId();
        
        //must has one task (potentially not claimed yet) as status Ready to processInstanceId = 1
        list = taskService.getTasksByStatusByProcessId(id, Arrays.asList(Status.Ready) , "en-UK");
        assertEquals(1, list.size());
        
        //processInstanceid = 2 does not have tasks
        list = taskService.getTasksByStatusByProcessId(2, Arrays.asList(Status.Ready) , "en-UK");
        assertEquals(0, list.size());
        
        //processInstanceid = 1 does not have tasks in status Reserved
        list = taskService.getTasksByStatusByProcessId(id, Arrays.asList(Status.Reserved) , "en-UK");
        assertEquals(0, list.size());
        
        //must has one task (to Jonh) as status Ready to processInstanceId = 1 and Task name = Task1
        list = taskService.getTasksByStatusByProcessIdByTaskName(id, Arrays.asList(Status.Ready) , "Task1", "en-UK");
        assertEquals(1, list.size());
        
        // processInstance = 1 does not have a Task name 'Task2' with Status Reserved
        list = taskService.getTasksByStatusByProcessIdByTaskName(id, Arrays.asList(Status.Ready) , "Task2", "en-UK");
        assertEquals(0, list.size());       
                
        assertProcessInstanceActive(processInstance.getId(), ksession);
        assertNodeTriggered(processInstance.getId(), "Start", "Task 1");
        
        // let john execute Task 1
        List<String> groups = new ArrayList<String>();
        groups.add("PM");       
        
        list = taskService.getTasksAssignedAsPotentialOwner("john",groups, "en-UK");
        TaskSummary task = list.get(0);
        System.out.println("John is claim the task " + task.getName());
        taskService.claim(task.getId(), "john", groups);
        System.out.println("John is executing task " + task.getName());
        taskService.start(task.getId(), "john");
        System.out.println("Task get id (john) =  "+ task.getId());
        taskService.complete(task.getId(), "john", null);

        assertNodeTriggered(processInstance.getId(), "Task 2");
        
        //must has one task (to Jonh) as status Completed to processInstanceId = 1
        list = taskService.getTasksByStatusByProcessId(id, Arrays.asList(Status.Completed) , "en-UK");
        assertEquals(1, list.size());
        
        //must returns two tasks, Task1 as completed and Task2 as Reserved
        list = taskService.getTasksByStatusByProcessId(id, Arrays.asList(Status.Completed, Status.Reserved) , "en-UK");
        assertEquals(2, list.size());
                
        // processInstance = 1 does not have a Task name 'Task1' with Status Reserved
        list = taskService.getTasksByStatusByProcessIdByTaskName(id, Arrays.asList(Status.Reserved) , "Task1", "en-UK");
        assertEquals(0, list.size());
        
        //must has one task (to Jonh) as status Ready to processInstanceId = 1 and Task name = Task2
        list = taskService.getTasksByStatusByProcessIdByTaskName(id, Arrays.asList(Status.Reserved) , "Task2", "en-UK");
        assertEquals(1, list.size());           
                        
        // let mary execute Task 2
        list = taskService.getTasksAssignedAsPotentialOwner("mary", "en-UK");
        task = list.get(0);
        System.out.println("Mary is executing task " + task.getName());
        taskService.start(task.getId(), "mary");
        System.out.println("Task get id (mary) =  "+ task.getId());
        taskService.complete(task.getId(), "mary", null);

        assertNodeTriggered(processInstance.getId(), "End");
        assertProcessInstanceCompleted(processInstance.getId(), ksession);
        
        //must has two tasks with status Completed to processInstanceId = 1
        list = taskService.getTasksByStatusByProcessId(id, Arrays.asList(Status.Completed) , "en-UK");
        assertEquals(2, list.size());       
        
        ksession.dispose();
    }
}
